@@ -9,18 +9,22 @@ from PyQt5.QtWidgets import QCheckBox, QPushButton, QLabel, QMainWindow, QSpinBo
 from PyQt5.QtWidgets import QApplication, QWidget, QHBoxLayout, QComboBox, QTabWidget
 from PyQt5.QtWidgets import QVBoxLayout
 from PyQt5.QtGui import QPixmap, QImage, QColor, QPainter
-from PyQt5.QtCore import Qt
+from PyQt5.QtCore import Qt, QPoint, QSize
 import sys
 import os
 
 import inspect
 import numpy as np
-from skimage.segmentation import mark_boundaries, flood_fill
+import math as m
+from skimage.segmentation import mark_boundaries, flood_fill, flood
 from skimage.util import invert
 from PIL import Image
 
 from ParticleSpy.segptcls import process
 from ParticleSpy.ParticleAnalysis import parameters
+from ParticleSpy.segimgs import ClusterTrained
+
+from sklearn.ensemble import RandomForestClassifier
 
 class Application(QMainWindow):
 
@@ -43,10 +47,12 @@ class Application(QMainWindow):
         self.tabs = QTabWidget()
         self.tab1 = QWidget()
         self.tab2 = QWidget()
+        self.tab3 = QWidget()
         
         # Add tabs
         self.tabs.addTab(self.tab1,"Auto")
         self.tabs.addTab(self.tab2,"Manual")
+        self.tabs.addTab(self.tab3,"Trainable")
 
         #self.central_widget = QWidget()               
         #self.setCentralWidget(self.central_widget)
@@ -55,6 +61,7 @@ class Application(QMainWindow):
         rightlay = QVBoxLayout()
         self.tab1.setLayout(lay)
 
+        #tab 1
         self.label = QLabel(self)
         qi = QImage(self.image.data, self.image.shape[1], self.image.shape[0], self.image.shape[1], QImage.Format_Grayscale8)
         pixmap = QPixmap(qi)
@@ -201,9 +208,67 @@ class Application(QMainWindow):
         tab2layout.addWidget(self.getarrayb)
         tab2layout.addStretch(1)
         self.tab2.setLayout(tab2layout)
+
+        #Tab 3
+
+        self.mask = np.zeros([512,512,3])
+        self.classifier = RandomForestClassifier(n_estimators=200)
+
+        lay3 = QHBoxLayout()
+        im_lay = QVBoxLayout()
+        button_lay = QVBoxLayout()
+        colour_lay = QHBoxLayout()
+
+
+        lay3.addLayout(button_lay)
+        lay3.addLayout(im_lay)
+
+        self.canvas2 = Canvas(self.pixmap2)
+        self.canvas2.setAlignment(Qt.AlignTop)
+        
+        for tool in brush_tools:
+            b = ToolButton(tool)
+            b.pressed.connect(lambda tool=tool: self.canvas2.changePen(tool))
+            b.setText(tool)
+            if tool == 'Freehand':
+                b.setChecked(True)
+            button_lay.addWidget(b)
+
+
+        for i in range(len(self.canvas2.colors)):
+            c = self.canvas2.colors[i]
+            b = QPaletteButton(c)
+            b.pressed.connect(lambda i=i: self.canvas2.set_pen_color(i))
+            if i== 0:
+                b.setChecked(True)
+            colour_lay.addWidget(b)
+
+        im_lay.addWidget(self.canvas2)
+
+        self.clear = QPushButton('Clear', self)
+        self.clear.clicked.connect(self.canvas2.clear)
+
+        self.bupdate = QPushButton('update', self)
+        self.bupdate.clicked.connect(self.train_update)
+
+        self.train = QPushButton('train classifier', self)
+        self.train.pressed.connect(self.train_classifier)
+
+        button_lay.addLayout(colour_lay)
+        button_lay.addWidget(self.bupdate)
+        button_lay.addWidget(self.train)
+        button_lay.addWidget(self.clear)
+
+        self.getarrayc = QPushButton('Save and Close',self)
+        self.getarrayc.clicked.connect(self.save_and_close)
+        
+        button_lay.addWidget(self.getarrayc)
+        self.tab3.setLayout(lay3)
+
+        
         
         self.show()
-        
+
     def updateLocalSize(self):
         if self.comboBox.currentText() == 'Niblack' or self.comboBox.currentText() == 'Sauvola' or self.comboBox.currentText() == 'Local':
             self.local_size.setEnabled(True)
@@ -340,72 +405,253 @@ class Application(QMainWindow):
             self.params.segment['threshold'] = "niblack"
         if str(self.comboBox.currentText()) == "Sauvola":
             self.params.segment['threshold'] = "sauvola"
+    
+    def train_update(self):
+        array = self.canvas2.array
+        self.mask = np.array(Image.fromarray(array).resize((self.image.shape[1],self.image.shape[0])))
+
+        print('updated labels')
+
+    def train_classifier(self):
+        
+        self.trained_mask, self.classifier = ClusterTrained(self.im_hs, self.mask, self.classifier)
+        self.canvas2.clear()
+        if self.trained_mask.any() != 0:
+            self.canvas2.drawLabels(self.trained_mask)
 
     def save_array(self):
         self.canvas.savearray(self.image)
 
+    def save_and_close(self):
+        array = self.canvas2.array
+        self.mask = np.array(Image.fromarray(array).resize((self.image.shape[1],self.image.shape[0])))
+        self.canvas2.savearray(self.image)
+        self.close()
+
+
+brush_tools = ['Freehand', 'Line', 'Polygon']
+
+class ToolButton(QPushButton):
+
+    def __init__(self, tool):
+        super().__init__()
+        self.setAutoExclusive(True)
+        self.setCheckable(True)
+
+class QPaletteButton(QPushButton):
+
+    def __init__(self, color):
+        super().__init__()
+        self.setAutoExclusive(True)
+        self.setCheckable(True)
+        self.setFixedSize(QSize(24,24))
+        self.color = color
+        self.setStyleSheet("background-color: %s;" % color)
 
 class Canvas(QLabel):
 
     def __init__(self,pixmap):
         super().__init__()
         self.setPixmap(pixmap)
+        self.OGpixmap = pixmap
+        self.lastpixmap = pixmap
+        """
+        self.tool_pixmap = QLabel()
+        tool_layer = QPixmap(self.size())
+        self.tool_pixmap.setPixmap(tool_layer)"""
+        
+        self.setMouseTracking(True)
+        self.first_click = None
+        self.last_click  = None
 
-        self.last_x, self.last_y = None, None
-        self.pen_color = QColor(255, 0, 0, 20)        
+        self.brush_tools = ['Freehand', 'Line', 'Polygon']
+        self.colors = ['#80FF0000', '#8000FF00', '#800000FF']
+        self.color_index = 0
+
+        self.pen_color = QColor(self.colors[0])
+        self.penType = self.brush_tools[0]
+        self.lineCount = 0
+
+        self.array = np.zeros((512,512,3),dtype=np.uint8)
 
     def set_pen_color(self, c):
-        self.pen_color = QColor(c)
-        
-    def mousePressEvent(self, e):
-        if e.button() == Qt.RightButton:
-            image = self.pixmap().toImage()
-            b = image.bits()
-            b.setsize(512 * 512 * 4)
-            arr = np.frombuffer(b, np.uint8).reshape((512, 512, 4))
-            
-            arr_test = arr[:,:,0]/arr[:,:,2]
-            
-            painted_arr = np.zeros_like(arr[:,:,0:3])
-            painted_arr[:,:,2][arr_test!=1] = 255
-            
-            painted_arr[:,:,2] = flood_fill(painted_arr[:,:,2],(e.y(),e.x()),255)
-            
-            qi = QImage(painted_arr.data, painted_arr.shape[1], painted_arr.shape[0], 3*painted_arr.shape[1], QImage.Format_RGB888)
-            pixmap = QPixmap(qi)
-            
-            painter = QPainter(self.pixmap())
-            painter.setOpacity(0.1)
+        self.color_index = c
+        self.pen_color = QColor(self.colors[self.color_index])
 
-            painter.drawPixmap(0, 0, pixmap)
-            painter.end()
-            self.update()
+    def changePen(self, brush):
+        self.last_click = None
+        self.lineCount = 0
+        self.penType = brush
+
+    def clear(self):
+
+        self.last_click = None
+        self.first_click = None
+        self.lineCount = 0
+        self.array = np.zeros((512,512,3), dtype=np.uint8)
+
+        painter = QPainter(self.pixmap())
+        painter.eraseRect(0,0,512,512)
+        painter.drawPixmap(0,0,self.OGpixmap)
+        painter.end()
+        self.update()
+
+    def drawLabels(self, thin_labels):
+
+        shape = thin_labels.shape
+        thicc_labels = np.zeros([shape[0], shape[1],4], dtype=np.uint8)
+        for c in range(1,4):
+            thicc_labels[:,:,c] = 255*(thin_labels == c)
+        thicc_labels[:,:,0] = 255*(thin_labels != 0)
+
+        thicc_labels = np.flip(thicc_labels, axis=2).copy()
+        qi = QImage(thicc_labels.data, thicc_labels.shape[1], thicc_labels.shape[0], 4*thicc_labels.shape[1], QImage.Format_ARGB32_Premultiplied)
+        
+        pixmap = QPixmap(qi)
+        pixmap = pixmap.scaled(512, 512, Qt.KeepAspectRatio)
+        painter = QPainter(self.pixmap())
+        painter.setOpacity(0.5)
+        painter.drawPixmap(0,0,pixmap)
+        painter.end()
+        self.update()
+
+    def lineDraw(self,pos1,pos2):
+        painter = QPainter(self.pixmap())
+        p = painter.pen()
+        p.setWidth(3)
+        p.setColor(self.pen_color)
+        painter.setPen(p)
+        painter.drawLine(pos1, pos2)
+        painter.end()
+        self.update()
+
+    def LineTool(self,e):
+        if self.lineCount == 0:
+            self.last_click = QPoint(e.x(),e.y())
+            self.lineCount = 1
+        else:
+            self.lineDraw(self.last_click,e.pos())
+            self.last_click = QPoint(e.x(),e.y())
+            self.lineCount = 0
+            midline = (self.last_click + e.pos())/2
+            self.flood(midline)
+
+    def PolyTool(self,e):
+        if self.lineCount == 0:
+            self.first_click = QPoint(e.x(),e.y())
+            self.last_click =  QPoint(e.x(),e.y())
+            self.lineCount = 1
+
+        elif self.lineCount == 1:
+            self.lineDraw(self.last_click,e.pos())
+            self.last_click = QPoint(e.x(),e.y())
+            self.lineCount += 1
+
+        elif self.lineCount > 1:
+            d_x, d_y = float(self.first_click.x()-e.x()), float(self.first_click.y() - e.y())
+            d_from_origin = m.sqrt((d_x)**2 + (d_y)**2)
+
+            if d_from_origin < 10:
+                
+                self.lineDraw(self.last_click, self.first_click)
+                self.last_click = None
+                self.first_click = None
+                self.lineCount = 0
+            else:
+                self.lineDraw(self.last_click, e.pos())
+                self.last_click = QPoint(e.x(),e.y())
+                self.lineCount += 1
+    
+    def flood(self, e):
+        image = self.pixmap().toImage()
+        b = image.bits()
+        b.setsize(512 * 512 * 4)
+        arr = np.frombuffer(b, np.uint8).reshape((512, 512, 4))
+        arr = arr.astype(np.int32)
+        arr = np.flip(arr, axis=2)
+
+        i = self.color_index + 1
+        arr_test = arr[:,:,i]-((arr[:,:,1]+ arr[:,:,2]+ arr[:,:,3])/3)
+        #arr test is not greyscale
+        
+        i = 2 - self.color_index
+        #painted_arr is BGRA
+        painted_arr = np.zeros_like(arr,dtype=np.uint8)
+        painted_arr[:,:,i][arr_test!=0] = 255
+        #this makes the drawn images the same as pen color
+
+        painted_arr[:,:,i] = 255*flood(painted_arr[:,:,i],(e.y(),e.x()))
+        #sets alpha from ith channel
+        painted_arr[:,:,3] = painted_arr[:,:,i]
+
+        #BGRA
+        qi = QImage(painted_arr.data, painted_arr.shape[1], painted_arr.shape[0], 4*painted_arr.shape[1], QImage.Format_ARGB32_Premultiplied)
+        pixmap = QPixmap(qi)
+        
+        painter = QPainter(self.pixmap())
+        painter.setOpacity(0.5)
+
+        painter.drawPixmap(0,0,pixmap)
+        painter.end()
+        self.update()
+        
+        #self.array saves RGB values
+        self.array += np.flip(painted_arr[:,:,:3], axis=2)
+
+    def mousePressEvent(self, e):
+
+        if e.button() == Qt.RightButton:
+            self.flood(e)
             
-            self.array = painted_arr[:,:,2]
+        
+        if e.button() ==Qt.LeftButton:
+
+            if self.penType == 'Line':
+                self.LineTool(e)
+                    
+            if self.penType == 'Polygon':
+                self.PolyTool(e)
+
 
     def mouseMoveEvent(self, e):
+        
         if e.buttons() == Qt.LeftButton:
-            if self.last_x is None: # First event.
-                self.last_x = e.x()
-                self.last_y = e.y()
+            if self.last_click is None: # First event.
+                self.last_click = QPoint(e.x(),e.y())
                 return # Ignore the first time.
             
-            painter = QPainter(self.pixmap())
-            p = painter.pen()
-            p.setWidth(4)
-            p.setColor(self.pen_color)
-            painter.setPen(p)
-            painter.drawLine(self.last_x, self.last_y, e.x(), e.y())
-            painter.end()
-            self.update()
-    
-            # Update the origin for next time.
-            self.last_x = e.x()
-            self.last_y = e.y()
+            if self.penType == 'Freehand':
+                self.lineDraw(self.last_click, e.pos())
+                # Update the origin for next time.
+                self.last_click = QPoint(e.x(),e.y())
+
+        """
+        if e.buttons() != Qt.LeftButton and (self.penType == 'Line' or self.penType == 'Polygon'):
+            if self.lineCount == 1:
+                painter = QPainter(self.tool_pixmap)
+                
+
+                #this just paints over the line behind with black ideally should isolate to new canvas
+
+                #draw new temp line
+                p = painter.pen()
+                p.setWidth(2)
+                p.pen_color = QColor(255, 255, 255, 255)
+                painter.setPen(p)
+
+                painter.drawLine(self.last_x_clicked, self.last_y_clicked, e.x(), e.y())
+                painter.end()
+                self.update()
+
+                self.last_x = e.x()
+                self.last_y = e.y()
+                """
+
+
 
     def mouseReleaseEvent(self, e):
-        self.last_x = None
-        self.last_y = None
+        if self.penType == 'Freehand':
+            self.last_click = None
         
     def savearray(self,image):
         resized = np.array(Image.fromarray(self.array).resize((image.shape[1],image.shape[0])))
